"""Utilities for authorization."""

import random
import string


def generate_nonce() -> str:
    """Generate a random nonce for oauth.

    Returns:
        str: The random nonce
    """
    return "".join(random.choices(string.ascii_uppercase + string.digits, k=16))  # noqa: S311



"""Utilities for the command line interface."""

import json
from collections.abc import Awaitable, Callable
from dataclasses import dataclass
from datetime import UTC, datetime
from enum import StrEnum
from functools import update_wrapper
from typing import TYPE_CHECKING, Any

import asyncclick as click
import yaml
from aiohttp.client_exceptions import ClientResponseError
from asyncclick.core import Context
from dateutil.parser import isoparse
from pygments import highlight
from pygments.formatters import TerminalFormatter
from pygments.lexers import JsonLexer, YamlLexer

if TYPE_CHECKING:
    from myskoda import MySkoda


async def handle_request(
    ctx: Context,
    func: Callable[..., Awaitable],
    *args: Any,  # noqa: ANN401
    **kwargs: Any,  # noqa: ANN401
) -> None:
    """Handle API requests and perform error handling."""
    try:
        result = await func(*args, **kwargs)
        if hasattr(result, "to_dict"):
            ctx.obj["print"](result.to_dict())
        else:
            ctx.obj["print"](result)
    except ClientResponseError as e:
        ctx.obj["print"]({"error": e.status, "message": e.message, "url": str(e.request_info.url)})


def mqtt_required[R](func: Callable[..., Awaitable[R]]) -> Callable[..., Awaitable[Awaitable[R]]]:
    """Enable MQTT before connecting to MySkoda."""

    @click.pass_context
    async def new_func(ctx: Context, *args, **kwargs) -> Awaitable[R]:  # noqa: ANN002, ANN003
        if ctx.obj["mqtt_disabled"]:
            return await ctx.invoke(func, *args, **kwargs)
        myskoda: MySkoda = ctx.obj["myskoda"]
        await myskoda.enable_mqtt()
        return await ctx.invoke(func, *args, **kwargs)

    return update_wrapper(new_func, func)


class Format(StrEnum):
    JSON = "json"
    YAML = "yaml"


def print_json(data: dict) -> None:
    print(highlight(json.dumps(data, indent=4), JsonLexer(), TerminalFormatter()))


def print_yaml(data: dict) -> None:
    print(highlight(yaml.dump(data), YamlLexer(), TerminalFormatter()))


def iso8601_datetime(
    _: click.Context,
    param: click.Parameter,
    value: str | None,
) -> datetime | None:
    if value is None:
        return None
    try:
        return isoparse(value)
    except (ValueError, TypeError) as e:
        err_str = f"{param.name} must be a valid ISO8601 datetime"
        raise click.BadParameter(err_str) from e


def simple_date(
    _: click.Context,
    param: click.Parameter,
    value: str | None,
) -> datetime | None:
    if value is None:
        return None
    try:
        return datetime.strptime(value, "%Y-%m-%d").replace(tzinfo=UTC)
    except (ValueError, TypeError) as e:
        err_str = f"{param.name} must be a valid YYYY-MM-DD date"
        raise click.BadParameter(err_str) from e


@dataclass
class MethodArgument:
    timestamp: datetime | None = None
    flag: bool | None = None
    text: str | None = None



"""Test helpers."""

import asyncio
import copy
import json
from collections.abc import AsyncIterator, Generator
from contextlib import AbstractAsyncContextManager
from pathlib import Path
from socket import socket
from types import TracebackType
from typing import Any
from unittest.mock import AsyncMock

import aiomqtt
import pytest
from aiohttp import ClientSession
from aioresponses import aioresponses

from myskoda.anonymize import ACCESS_TOKEN
from myskoda.auth.authorization import Authorization
from myskoda.mqtt import MySkodaMqttClient
from myskoda.myskoda import MySkoda, MySkodaAuthorization
from myskoda.rest_api import RestApi

FIXTURES_DIR = Path(__file__).parent / "fixtures"
TRACE_ID = "7a59299d06535a6756d10e96e0c75ed3"
REQUEST_ID = "b9bc1258-2d0c-43c2-8d67-44d9f6c8cb9f"


class FakeAuthorization(Authorization):
    @property
    def client_id(self) -> str | None:
        return "1"

    @property
    def redirect_uri(self) -> str | None:
        return "https://fake/auth/redirect"

    @property
    def base_url(self) -> str | None:
        return "https://fake/auth"

    async def get_access_token(self) -> str:
        return "access_token"


class SimpleAsyncIterator:
    """A simple AsyncIterator which returns any type of items.

    Items can be set in __init__ or later added with the items property.
    """

    def __init__(self, items: list[Any]) -> None:
        self._new_items_available = asyncio.Event()
        self._items = []
        if items:
            self.items = items

    def __aiter__(self):  # noqa: ANN204
        return self

    @property
    def items(self) -> list[Any]:
        return self._items

    @items.setter
    def items(self, value: list[Any]) -> None:
        self._items = copy.deepcopy(value)  # copy to avoid popping of the passed value
        self._new_items_available.set()

    async def __anext__(self):  # noqa: ANN204
        """Block until (new) items are available."""
        await self._new_items_available.wait()
        next_item = self._items.pop()
        if not self._items:
            self._new_items_available.clear()
        return next_item


class FakeMqttClientWrapper(AbstractAsyncContextManager):
    """Fake aiomqtt.Client wrapper to use in tests.

    Can be initiatlized with a list of aiomqtt.Message's which will be available immediatally.
    Additional messages can be injected later using set_messages().
    """

    def __init__(self, messages: list[aiomqtt.Message]) -> None:
        self._aiter = SimpleAsyncIterator(messages)

    async def __aexit__(
        self,
        exc_type: type[BaseException] | None,
        exc: BaseException | None,
        tb: TracebackType | None,
    ) -> None:
        pass

    def set_messages(self, messages: list[aiomqtt.Message]) -> None:
        self._aiter.items = messages

    @property
    def messages(self) -> AsyncIterator:
        return self._aiter

    async def subscribe(self, topic: str) -> None:
        print(f"Fake subscribed to topic {topic}")

    def update_username_password(self, username: str, password: str) -> None:
        print(f"Fake updated username/password as {username}/{password}")


@pytest.fixture
async def fake_authorization() -> FakeAuthorization:
    """Return a fake Authorization instance."""
    async with ClientSession() as session:
        return FakeAuthorization(session)


@pytest.fixture
def fake_mqtt_client_wrapper() -> FakeMqttClientWrapper:
    """Return a fake AbstractMqttClientWrapper instance."""
    return FakeMqttClientWrapper(messages=[])


@pytest.fixture
async def myskoda_mqtt_client(
    fake_authorization: FakeAuthorization, fake_mqtt_client_wrapper: FakeMqttClientWrapper
) -> AsyncIterator[MySkodaMqttClient]:
    """Return a MySkodaMqttClient instance to use in tests."""
    mqtt_client = MySkodaMqttClient(
        authorization=fake_authorization, mqtt_client=fake_mqtt_client_wrapper
    )
    yield mqtt_client
    await mqtt_client.disconnect()


@pytest.fixture
def responses() -> Generator[aioresponses]:
    """Return aioresponses fixture."""
    with aioresponses() as mocked_responses:
        yield mocked_responses


@pytest.fixture
async def api() -> AsyncIterator[RestApi]:
    """Return rest api."""
    async with ClientSession() as session:
        authorization = MySkodaAuthorization(session)
        api = RestApi(session, authorization)
        api.authorization.get_access_token = AsyncMock()
        yield api


def random_port() -> int:
    with socket() as sock:
        sock.bind(("", 0))
        return sock.getsockname()[1]


def mock_default_routes(responses: aioresponses) -> None:
    responses.get(
        url="https://mysmob.api.connect.skoda-auto.cz/api/v1/users",
        body=(FIXTURES_DIR / "mqtt" / "user.json").read_text(),
    )
    responses.get(
        url="https://mysmob.api.connect.skoda-auto.cz/api/v2/garage?connectivityGenerations=MOD1&connectivityGenerations=MOD2&connectivityGenerations=MOD3&connectivityGenerations=MOD4",
        body=(FIXTURES_DIR / "mqtt" / "vehicles.json").read_text(),
    )


@pytest.fixture
async def myskoda(
    responses: aioresponses, myskoda_mqtt_client: MySkodaMqttClient
) -> AsyncIterator[MySkoda]:
    """Return rest api."""
    async with ClientSession() as session:
        mock_default_routes(responses)
        myskoda = MySkoda(session, mqtt_enabled=False)
        myskoda.mqtt = myskoda_mqtt_client
        myskoda.authorization.get_access_token = AsyncMock(return_value=ACCESS_TOKEN)
        myskoda.authorization.authorize = AsyncMock()
        await myskoda.connect("user@example.com", "password")
        yield myskoda
        await myskoda.disconnect()


def create_aiomqtt_message(topic: str, operation: str) -> aiomqtt.Message:
    payload = json.dumps(
        {
            "version": 1,
            "operation": operation,
            "status": "COMPLETED_SUCCESS",
            "traceId": TRACE_ID,
            "requestId": REQUEST_ID,
        }
    ).encode("utf-8")

    return aiomqtt.Message(
        topic=topic,
        payload=payload,
        qos=1,
        retain=False,
        mid=1,
        properties=None,
    )



"""Models for responses of api/v2/auxiliary_heating endpoint."""

from dataclasses import dataclass, field
from datetime import datetime
from enum import StrEnum
from typing import Any

from mashumaro import field_options
from mashumaro.config import TO_DICT_ADD_BY_ALIAS_FLAG, TO_DICT_ADD_OMIT_NONE_FLAG, BaseConfig
from mashumaro.mixins.orjson import DataClassORJSONMixin

from .air_conditioning import (
    AirConditioningTimer,
    HeaterSource,
    OutsideTemperature,
    TargetTemperature,
)
from .common import BaseResponse


class AuxiliaryState(StrEnum):
    HEATING_AUXILIARY = "HEATING_AUXILIARY"
    INVALID = "INVALID"
    OFF = "OFF"
    PREHEATING = "PREHEATING"
    UNSUPPORTED = "UNSUPPORTED"
    VENTILATION = "VENTILATION"


class AuxiliaryStartMode(StrEnum):
    HEATING = "HEATING"
    VENTILATION = "VENTILATION"
    INVALID = "INVALID"


@dataclass
class AuxiliaryConfig(DataClassORJSONMixin):
    """Configuration needed for starting auxiliary heater."""

    target_temperature: TargetTemperature | None = field(
        default=None, metadata=field_options(alias="targetTemperature")
    )
    duration_in_seconds: int | None = field(
        default=None, metadata=field_options(alias="durationInSeconds")
    )
    heater_source: HeaterSource | None = field(
        default=None, metadata=field_options(alias="heaterSource")
    )
    start_mode: AuxiliaryStartMode | None = field(
        default=None, metadata=field_options(alias="startMode")
    )

    class Config(BaseConfig):
        """Configuration for serialization and deserialization.."""

        code_generation_options = [  # noqa: RUF012
            TO_DICT_ADD_BY_ALIAS_FLAG,
            TO_DICT_ADD_OMIT_NONE_FLAG,
        ]

    def __pre_serialize__(self) -> "AuxiliaryConfig":
        """Round target temperature before serialization to 0.5."""
        if self.target_temperature is not None:
            self.target_temperature.temperature_value = (
                round(self.target_temperature.temperature_value * 2) / 2
            )
        return self


@dataclass
class AuxiliaryHeatingTimer(AirConditioningTimer):
    """Timer for auxiliary heating."""


@dataclass
class AuxiliaryHeating(BaseResponse):
    """Information related to auxiliary heating."""

    timers: list[AuxiliaryHeatingTimer]
    errors: list[Any]
    state: AuxiliaryState | None = field(default=None, metadata=field_options(alias="state"))
    start_mode: AuxiliaryStartMode | None = field(
        default=None, metadata=field_options(alias="startMode")
    )
    duration_in_seconds: int | None = field(
        default=None, metadata=field_options(alias="durationInSeconds")
    )
    target_temperature: TargetTemperature | None = field(
        default=None, metadata=field_options(alias="targetTemperature")
    )
    car_captured_timestamp: datetime | None = field(
        default=None, metadata=field_options(alias="carCapturedTimestamp")
    )
    estimated_date_time_to_reach_target_temperature: datetime | None = field(
        default=None, metadata=field_options(alias="estimatedDateTimeToReachTargetTemperature")
    )
    outside_temperature: OutsideTemperature | None = field(
        default=None, metadata=field_options(alias="outsideTemperature")
    )



"""Unit tests for utilities."""

import asyncio
from datetime import UTC, datetime, timedelta, timezone

import pytest

from myskoda.utils import async_debounce, to_iso8601


@pytest.mark.asyncio
async def test_async_debounce() -> None:
    """Test the async_debounce decorator.

    Debounced function:
    - Only executes once when called multiple times within 'wait'.
    - Executes twice when called two times with a delay > 'wait'.
    """
    test_val = 0

    @async_debounce(wait=0.2)
    async def increment() -> None:
        nonlocal test_val
        test_val += 1

    await increment()
    await increment()
    await asyncio.sleep(0.1)
    await increment()
    await asyncio.sleep(0.3)
    assert test_val == 1

    await increment()
    await asyncio.sleep(0.3)
    await increment()
    await asyncio.sleep(0.3)
    assert test_val == 3  # noqa: PLR2004


@pytest.mark.asyncio
async def test_async_debounce_immediate() -> None:
    """Test the async_debounce decorator when immediate is True.

    Debounced function:
    - Executes immediatally the first time called.
    - Subsequent calls are executed (once) after wait time.
    """
    test_val = 0

    @async_debounce(wait=0.2, immediate=True)
    async def increment() -> None:
        nonlocal test_val
        test_val += 1

    await increment()
    assert test_val == 1
    await increment()
    await increment()
    await asyncio.sleep(0.3)
    assert test_val == 2  # noqa: PLR2004


@pytest.mark.asyncio
async def test_async_debounce_immediate_noqueue() -> None:
    """Test the async_debounce decorator when immediate is True and queue is False.

    Debounced function:
    - Executes immediatally the first time called.
    - Subsequent calls are executed (once) after wait time.
    """
    test_val = 0

    @async_debounce(wait=0.2, immediate=True, queue=False)
    async def increment() -> None:
        nonlocal test_val
        test_val += 1

    await increment()
    assert test_val == 1
    await increment()
    await increment()
    await asyncio.sleep(0.3)
    assert test_val == 1


def test_to_is8601_utc_datetime() -> None:
    """Test the to_iso8601 function with UTC datetime input.

    Make sure time stays the same and end with Z
    """
    dt = datetime(2025, 9, 10, 10, 0, 0, tzinfo=UTC)
    result = to_iso8601(dt)
    assert result == "2025-09-10T10:00:00Z"


def test_to_iso8601_offset_datetime() -> None:
    """Test the to_iso8601 function with non-UTC datetime input.

    Make sure time stays the same UTC value and end with Z
    """
    tz = timezone(timedelta(hours=2))
    dt = datetime(2025, 9, 10, 12, 0, 0, tzinfo=tz)
    result = to_iso8601(dt)
    assert result.startswith("2025-09-10T10:00")
    assert result.endswith("Z")



"""Commands dealing with reading data from the Rest API."""

from datetime import datetime
from typing import TYPE_CHECKING, Any

import asyncclick as click
from asyncclick.core import Context

from myskoda.cli.utils import MethodArgument, handle_request, iso8601_datetime, simple_date

if TYPE_CHECKING:
    from myskoda.myskoda import MySkoda


@click.command()
@click.pass_context
async def list_vehicles(ctx: Context) -> None:
    """Print a list of all vehicle identification numbers associated with the account."""
    await handle_request(ctx, ctx.obj["myskoda"].list_vehicle_vins)


@click.command()
@click.argument("vin")
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def info(ctx: Context, vin: str, anonymize: bool) -> None:
    """Print info for the specified vin."""
    await handle_request(ctx, ctx.obj["myskoda"].get_info, vin, anonymize)


@click.command()
@click.argument("vin")
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def status(ctx: Context, vin: str, anonymize: bool) -> None:
    """Print current status for the specified vin."""
    await handle_request(ctx, ctx.obj["myskoda"].get_status, vin, anonymize)


@click.command()
@click.argument("vin")
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def air_conditioning(ctx: Context, vin: str, anonymize: bool) -> None:
    """Print current status about air conditioning."""
    await handle_request(ctx, ctx.obj["myskoda"].get_air_conditioning, vin, anonymize)


@click.command()
@click.argument("vin")
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def auxiliary_heating(ctx: Context, vin: str, anonymize: bool) -> None:
    """Print current status about auxiliary heating."""
    await handle_request(ctx, ctx.obj["myskoda"].get_auxiliary_heating, vin, anonymize)


@click.command()
@click.argument("vin")
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def positions(ctx: Context, vin: str, anonymize: bool) -> None:
    """Print the vehicle's current position."""
    await handle_request(ctx, ctx.obj["myskoda"].get_positions, vin, anonymize)


@click.command()
@click.argument("vin")
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def parking_position(ctx: Context, vin: str, anonymize: bool) -> None:
    """Print the vehicle's current position."""
    await handle_request(ctx, ctx.obj["myskoda"].get_parking_position, vin, anonymize)


@click.command()
@click.argument("vin")
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def health(ctx: Context, vin: str, anonymize: bool) -> None:
    """Print the vehicle's mileage."""
    await handle_request(ctx, ctx.obj["myskoda"].get_health, vin, anonymize)


@click.command()
@click.argument("vin")
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def charging(ctx: Context, vin: str, anonymize: bool) -> None:
    """Print the vehicle's current charging state."""
    await handle_request(ctx, ctx.obj["myskoda"].get_charging, vin, anonymize)


@click.command()
@click.argument("vin")
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def charging_profiles(ctx: Context, vin: str, anonymize: bool) -> None:
    """Print the vehicle's charging profiles."""
    await handle_request(ctx, ctx.obj["myskoda"].get_charging_profiles, vin, anonymize)


@click.command()
@click.argument("vin")
@click.option("--start", "start", help="ISO8601 compatible start date", callback=iso8601_datetime)
@click.option("--end", "end", help="ISO8601 compatible end date", callback=iso8601_datetime)
@click.option(
    "--cursor",
    "cursor",
    help="ISO8601 compatible cursor date. When this is set, --start and --end are ignored",
    callback=iso8601_datetime,
)
@click.option(
    "--limit",
    "limit",
    help="Maximum amount of sessions in the result",
    type=click.IntRange(1, 99),
    default=50,
)
@click.pass_context
async def charging_history(  # noqa: PLR0913
    ctx: Context,
    vin: str,
    start: datetime | None = None,
    end: datetime | None = None,
    cursor: datetime | None = None,
    limit: int = 50,
) -> None:
    """Print the vehicle's charging history."""

    if cursor is None and (start is None or end is None):
        err_msg = "Either --cursor must be set, or both --start and --end must be provided."
        raise click.BadParameter(err_msg)

    kwargs: dict[str, Any] = {
        k: v
        for k, v in (
            ("vin", vin),
            ("start", start),
            ("end", end),
            ("cursor", cursor),
            ("limit", limit),
        )
        if v is not None
    }

    await handle_request(ctx, ctx.obj["myskoda"].get_charging_history, **kwargs)


@click.command()
@click.argument("vin")
@click.pass_context
async def all_charging_sessions(ctx: Context, vin: str) -> None:
    """Print all vehicles charging sessions."""
    await handle_request(ctx, ctx.obj["myskoda"].get_all_charging_sessions, vin)


@click.command()
@click.argument("vin")
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def maintenance(ctx: Context, vin: str, anonymize: bool) -> None:
    """Print the vehicle's maintenance information."""
    await handle_request(ctx, ctx.obj["myskoda"].get_maintenance, vin, anonymize)


@click.command()
@click.argument("vin")
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def maintenance_report(ctx: Context, vin: str, anonymize: bool) -> None:
    """Print the vehicle's maintenance report."""
    await handle_request(ctx, ctx.obj["myskoda"].get_maintenance_report, vin, anonymize)


@click.command()
@click.argument("vin")
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def driving_range(ctx: Context, vin: str, anonymize: bool) -> None:
    """Print the vehicle's estimated driving range information."""
    await handle_request(ctx, ctx.obj["myskoda"].get_driving_range, vin, anonymize)


@click.command()
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def user(ctx: Context, anonymize: bool) -> None:
    """Print information about currently logged in user."""
    await handle_request(ctx, ctx.obj["myskoda"].get_user, anonymize)


@click.command()
@click.argument("vin")
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def trip_statistics(ctx: Context, vin: str, anonymize: bool) -> None:
    """Print the last trip statics."""
    await handle_request(ctx, ctx.obj["myskoda"].get_trip_statistics, vin, anonymize)


@click.command()
@click.argument("vin")
@click.option(
    "--start", "start", help="start date in format YYYY-MM-DD (inclusive)", callback=simple_date
)
@click.option(
    "--end", "end", help="end date in format YYYY-MM-DD (exclusive)", callback=simple_date
)
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def single_trip_statistics(
    ctx: Context,
    vin: str,
    anonymize: bool,
    start: datetime | None = None,
    end: datetime | None = None,
) -> None:
    """Retrieve detailed statistics about past trips.
    If you want to filter by date, provide both start and end date."""
    if (start is None) ^ (end is None):
        err_msg = "Both --start and --end must be provided."
        raise click.BadParameter(err_msg)

    kwargs: dict[str, datetime | bool | str] = {
        k: v
        for k, v in (
            ("vin", MethodArgument(text=vin).text),
            ("start", MethodArgument(timestamp=start).timestamp),
            ("end", MethodArgument(timestamp=end).timestamp),
            ("anonymize", MethodArgument(flag=anonymize).flag),
        )
        if v is not None
    }
    await handle_request(ctx, ctx.obj["myskoda"].get_single_trip_statistics, **kwargs)


@click.command()
@click.argument("vin")
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def connection_status(ctx: Context, vin: str, anonymize: bool) -> None:
    """Get the vehicle connection state."""
    await handle_request(ctx, ctx.obj["myskoda"].get_connection_status, vin, anonymize)


@click.command()
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def garage(ctx: Context, anonymize: bool) -> None:
    """Print garage information (list of vehicles with limited information)."""
    await handle_request(ctx, ctx.obj["myskoda"].rest_api.get_garage, anonymize)


@click.command()
@click.option("spin", "--spin", type=str, required=True)
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def verify_spin(ctx: Context, spin: str, anonymize: bool) -> None:
    """Verify S-PIN."""
    await handle_request(ctx, ctx.obj["myskoda"].verify_spin, spin, anonymize)


@click.command()
@click.argument("vin")
@click.option("anonymize", "--anonymize", help="Strip all personal data.", is_flag=True)
@click.pass_context
async def departure_timers(ctx: Context, vin: str, anonymize: bool) -> None:
    """Get all departure timers."""
    await handle_request(ctx, ctx.obj["myskoda"].get_departure_timers, vin, anonymize)


@click.command()
@click.pass_context
async def auth(ctx: Context) -> None:
    """Extract the auth token."""
    myskoda: MySkoda = ctx.obj["myskoda"]
    print(await myskoda.get_auth_token())



"""Tests for trip statistics models."""

import json
from pathlib import Path

from myskoda.models.trip_statistics import SingleTrips, TripStatistics

FIXTURES_DIR = Path(__file__).parent.joinpath("fixtures")


def test_parse_single_trips() -> None:
    """Test parsing SingleTrips from real-world fixture."""
    fixture_path = FIXTURES_DIR.joinpath("superb/single-trips-iV.json")
    json_data = fixture_path.read_text()

    parsed = SingleTrips.from_json(json_data)

    assert parsed is not None
    assert parsed.daily_trips is not None
    assert len(parsed.daily_trips) > 0

    raw_data = json.loads(json_data)
    assert parsed.daily_trips[0].date == raw_data["dailyTrips"][0]["date"]

    if parsed.daily_trips[0].trips:
        assert (
            parsed.daily_trips[0].trips[0].end_time
            == raw_data["dailyTrips"][0]["trips"][0]["endTime"]
        )


def test_parse_trip_statistics() -> None:
    """Test parsing TripStatistics from real-world fixture."""
    fixture_path = FIXTURES_DIR.joinpath("superb/trip-statistics-iV.json")
    json_data = fixture_path.read_text()

    parsed = TripStatistics.from_json(json_data)

    assert parsed is not None

    raw_data = json.loads(json_data)
    assert parsed.overall_mileage_in_km == raw_data.get("overallMileageInKm")
    assert parsed.overall_average_speed_in_kmph == raw_data.get("overallAverageSpeedInKmph")



"""Unit tests for myskoda.auth."""

from json import dumps
from pathlib import Path

import aiohttp
import pytest
from aioresponses import aioresponses

from myskoda.anonymize import USER_ID
from myskoda.const import BASE_URL_IDENT, BASE_URL_SKODA, CLIENT_ID
from myskoda.myskoda import MySkodaAuthorization

FIXTURES_DIR = Path(__file__).parent.joinpath("fixtures")


def fixture(filename: str) -> str:
    with FIXTURES_DIR.joinpath(filename).open() as file:
        return file.read()


def redirect_loop(responses: aioresponses, urls: list[str]) -> None:
    responses.post(
        url=urls[0],
        status=301,
        headers={
            "Location": urls[1],
        },
    )
    for index in range(1, len(urls) - 1):
        responses.get(
            url=urls[index],
            status=301,
            headers={
                "Location": urls[index + 1],
            },
        )


@pytest.mark.asyncio
async def test_get_tokens(responses: aioresponses) -> None:
    nonce = "abcdefghabcdefgh"
    relay_state = "d865b506bd6759b20e832c8c692c5ca2669ebd27"
    user_id = USER_ID
    hmac = "575452461e126b1873f4655918e14d0ba1a40622b768438fe4dd6d9579bc170c"

    def generate_nonce() -> str:
        return nonce

    responses.get(
        url="https://identity.vwgroup.io/oidc/v1/authorize?client_id=7f045eee-7003-4379-9968-9355ed2adb06%2540apps_vw-dilab_com&code_challenge=GB77VCZkQqwqOPgKuV1f4TxM4_OxLWfBxprenr3kfE0&code_challenge_method=s256&nonce=abcdefghabcdefgh&prompt=login&redirect_uri=myskoda%253A%252F%252Fredirect%252Flogin%252F&response_type=code&scope=address+badge+birthdate+cars+driversLicense+dealers+email+mileage+mbb+nationalIdentifier+openid+phone+profession+profile+vin",
        status=200,
        body=fixture("auth/signin.html"),
    )

    responses.post(
        url="https://identity.vwgroup.io/signin-service/v1/7f045eee-7003-4379-9968-9355ed2adb06@apps_vw-dilab_com/login/identifier",
        status=200,
        body=fixture("auth/identifier.html"),
    )

    jwt_login = "eyJ0eXAiOiI0ODEyODgzZi05Y2FiLTQwMWMtYTI5OC0wZmEyMTA5Y2ViY2EiLCJhbGciOiJSUzI1NiJ9.eyJzdWIiOiJiOGJjMTI2Yy1lZTM2LTQwMmItODcyMy0yYzFjM2RmZjhkZWMiLCJhdWQiOiI3ZjA0NWVlZS03MDAzLTQzNzktOTk2OC05MzU1ZWQyYWRiMDZAYXBwc192dy1kaWxhYl9jb20iLCJhY3IiOiJodHRwczovL2lkZW50aXR5LnZ3Z3JvdXAuaW8vYXNzdXJhbmNlL2xvYS0yIiwic2NwIjoiYWRkcmVzcyBiYWRnZSBiaXJ0aGRhdGUgY2FycyBkcml2ZXJzTGljZW5zZSBkZWFsZXJzIGVtYWlsIG1pbGVhZ2UgbWJiIG5hdGlvbmFsSWRlbnRpZmllciBvcGVuaWQgcGhvbmUgcHJvZmVzc2lvbiBwcm9maWxlIHZpbiIsImFhdCI6ImlkZW50aXR5a2l0IiwiaXNzIjoiaHR0cHM6Ly9pZGVudGl0eS52d2dyb3VwLmlvIiwianR0IjoiYXV0aG9yaXphdGlvbl9jb2RlIiwiZXhwIjoxNzI4NTc2NDEwLCJpYXQiOjE3Mjg1NzYxMTAsIm5vbmNlIjoiYXNkZmdoYXNkZmdoIiwianRpIjoiOTdkODRiN2YtYzRhOC00NDcyLTllZjEtMzkyZWU4MTkwMzUwIn0.RllwxrkQTm8Z-2tIA4fGiBEP-b77QHLWzHAROhYBKVRne4s-aQdWtHFadIp0ikX6yyExeYYOVzcKBZf5FFnlaFjZB7hf5fVhfcp_TIbgs0Do_4cVz8wEFgYLFtBImeg9QhMfv11kYFEwkDBAtCeVsc6wdefIhzZrdszygW83wHN2hYuuyQYK0TWBC9yDsyQmEUuzqMkRgg0O_FdVYavJlL-orydiXn1DZiyCBfB4OHOQmbiCQr5CqMpgXV6dkE2WYi0w9NnxhtkWe-RXNpO4QzJkuMIJ1hOILIto5LM50GvO61M9hAcbp8fdx_WrTnZ1ENlsLBCojOHrvSdY4RMelA"  # noqa: E501

    redirect_loop(
        responses,
        [
            f"{BASE_URL_IDENT}/signin-service/v1/{CLIENT_ID}/login/authenticate",
            f"{BASE_URL_IDENT}/oidc/v1/oauth/sso?HMAC={hmac}&clientId=7f045eee-7003-4379-9968-9355ed2adb06%2540apps_vw-dilab_com&relayState={relay_state}&userId={user_id}",
            f"{BASE_URL_IDENT}/signin-service/v1/consent/users/{user_id}/{CLIENT_ID}?scopes=address%20badge%20birthdate%20cars%20driversLicense%20dealers%20email%20mileage%20mbb%20nationalIdentifier%20openid%20phone%20profession%20profile%20vin&relayState={relay_state}&callback={BASE_URL_IDENT}/oidc/v1/oauth/client/callback&hmac={hmac}",
            f"{BASE_URL_IDENT}/oidc/v1/oauth/client/callback/success?user_id={user_id}&client_id={CLIENT_ID}&scopes=address%20badge%20birthdate%20cars%20driversLicense%20dealers%20email%20mileage%20mbb%20nationalIdentifier%20openid%20phone%20profession%20profile%20vin&consentedScopes=address%20badge%20birthdate%20cars%20driversLicense%20dealers%20email%20mileage%20mbb%20nationalIdentifier%20openid%20phone%20profession%20profile%20vin&relayState={relay_state}&hmac={hmac}",
            f"myskoda://redirect/login/?code={jwt_login}",
        ],
    )

    access_token = "eyJ0eXAiOiI0ODEyODgzZi05Y2FiLTQwMWMtYTI5OC0wZmEyMTA5Y2ViY2EiLCJhbGciOiJSUzI1NiJ9.eyJzdWIiOiJiOGJjMTI2Yy1lZTM2LTQwMmItODcyMy0yYzFjM2RmZjhkZWMiLCJhdWQiOiI3ZjA0NWVlZS03MDAzLTQzNzktOTk2OC05MzU1ZWQyYWRiMDZAYXBwc192dy1kaWxhYl9jb20iLCJzY3AiOiJhZGRyZXNzIGJhZGdlIGJpcnRoZGF0ZSBjYXJzIGRyaXZlcnNMaWNlbnNlIGRlYWxlcnMgZW1haWwgbWlsZWFnZSBtYmIgbmF0aW9uYWxJZGVudGlmaWVyIG9wZW5pZCBwaG9uZSBwcm9mZXNzaW9uIHByb2ZpbGUgdmluIiwiYWF0IjoiaWRlbnRpdHlraXQiLCJpc3MiOiJodHRwczovL2lkZW50aXR5LnZ3Z3JvdXAuaW8iLCJqdHQiOiJhY2Nlc3NfdG9rZW4iLCJleHAiOjE3Mjg1OTkzMTUsImlhdCI6MTcyODU5NTcxNSwibGVlIjpbIlNLT0RBIl0sImp0aSI6IjA4ZDE0NmQ2LWUyMGItNDFkNS1hNmQ1LWUzNGY5NTBmZGUyZCJ9.WexOySWyj4caq7X731273RnB7HYmZ8sh5LdXRrl9w7gEGfOuwywkNaw66QUhImZTWv-bEO3c9dyxx7Gy2_qKK74bnwJCe9cW68G6UZCw5bUvbNl4Z7k2_xX9ko14r8m4vfIksq19qTrTCW5vELbp37EC1w0EZ1BdVVCyte-VdeiVTsPwZkrMjrBBGq_PWu-kQgyWwrh2CEhSN5BR9QybDe10_-ngHF4L9ulbgp01YT4mSvHBJXwNplb3YkM9MAFBuOGe1U9F05GvP7g3JLQeN7cLEODpjPDrsg4JhLldbsZcemS4X304bhpnbWkAzPohSrzRHjb7ny64eEnN16oAdg"  # noqa: E501, S105
    refresh_token = "eyJ0eXAiOiI0ODEyODgzZi05Y2FiLTQwMWMtYTI5OC0wZmEyMTA5Y2ViY2EiLCJhbGciOiJSUzI1NiJ9.eyJzdWIiOiJiOGJjMTI2Yy1lZTM2LTQwMmItODcyMy0yYzFjM2RmZjhkZWMiLCJhdWQiOiI3ZjA0NWVlZS03MDAzLTQzNzktOTk2OC05MzU1ZWQyYWRiMDZAYXBwc192dy1kaWxhYl9jb20iLCJhY3IiOiJodHRwczovL2lkZW50aXR5LnZ3Z3JvdXAuaW8vYXNzdXJhbmNlL2xvYS0yIiwic2NwIjoiYWRkcmVzcyBiYWRnZSBiaXJ0aGRhdGUgY2FycyBkcml2ZXJzTGljZW5zZSBkZWFsZXJzIGVtYWlsIG1pbGVhZ2UgbWJiIG5hdGlvbmFsSWRlbnRpZmllciBvcGVuaWQgcGhvbmUgcHJvZmVzc2lvbiBwcm9maWxlIHZpbiIsImFhdCI6ImlkZW50aXR5a2l0IiwiaXNzIjoiaHR0cHM6Ly9pZGVudGl0eS52d2dyb3VwLmlvIiwianR0IjoicmVmcmVzaF90b2tlbiIsImV4cCI6MTc0NDE0NzcxNSwiaWF0IjoxNzI4NTk1NzE1LCJqdGkiOiIwOGQxNDZkNi1lMjBiLTQxZDUtYTZkNS1lMzRmOTUwZmRlMmQifQ.HaGOdGak6Ce6jJikRM8czRzDjdCpkQjJgivE7kUgtdyCF4jh4KdwNuQ3VyOFgbzj86rkpbsVGhAmhgo3VzgHqg_R--iATRZ-7m0dufzThBeI69X_7XSjisUfNEvByX9ZIMXdH1Vo0yixKmQnTDqqgTNsA6BiwKskdKwOLeiYatwgt8uLTsMaZmc4BtR90db-dlLunvMy5_PlA7NxKBtQCeMET15FAgmIsWZWb8R7SYmap9gjnptgoWQMb4TPaS7A01E8UWpq6UqR3juaMFhqSarHFOiFT_fV1e_KTUqEch08iTkWJcyXEPeWSIiMqVLvKw0nn1z5JqlGK_-msreYPQ"  # noqa: E501, S105
    id_token = "eyJ0eXAiOiI0ODEyODgzZi05Y2FiLTQwMWMtYTI5OC0wZmEyMTA5Y2ViY2EiLCJhbGciOiJSUzI1NiJ9.eyJhdF9oYXNoIjoiaFNDV0dtMDRjYVhrcGdSSkFvRVQzUSIsInN1YiI6ImI4YmMxMjZjLWVlMzYtNDAyYi04NzIzLTJjMWMzZGZmOGRlYyIsImVtYWlsX3ZlcmlmaWVkIjp0cnVlLCJjb3IiOiJERSIsImlzcyI6Imh0dHBzOi8vaWRlbnRpdHkudndncm91cC5pbyIsImp0dCI6ImlkX3Rva2VuIiwidHlwZSI6ImlkZW50aXR5Iiwibm9uY2UiOiI2WUFHSTVVUkUwWThQN0xDIiwibGVlIjpbIlNLT0RBIl0sImF1ZCI6WyI3ZjA0NWVlZS03MDAzLTQzNzktOTk2OC05MzU1ZWQyYWRiMDZAYXBwc192dy1kaWxhYl9jb20iLCJodHRwczovL2FwaS52YXMuZXUuZHAxNS52d2ctY29ubmVjdC5jb20iLCJodHRwczovL2FwaS52YXMuZXUud2NhcmRwLmlvIiwiaHR0cHM6Ly9wcm9kLmVjZS5nYXV0aC12d2FjLmNvbSIsIlZXR01CQjAxQ05BUFAxIiwiVldHTUJCMDFERUxJVjEiXSwiYWNyIjoiaHR0cHM6Ly9pZGVudGl0eS52d2dyb3VwLmlvL2Fzc3VyYW5jZS9sb2EtMiIsInVwZGF0ZWRfYXQiOjE3MTY5NzI2NjY0ODYsImFhdCI6ImlkZW50aXR5a2l0IiwiZXhwIjoxNzI4NTk5MzE1LCJpYXQiOjE3Mjg1OTU3MTUsImp0aSI6IjA4ZDE0NmQ2LWUyMGItNDFkNS1hNmQ1LWUzNGY5NTBmZGUyZCIsImVtYWlsIjoidXNlckBleGFtcGxlLmNvbSJ9.zfCXYJJg9eJ3L_OKnjix7bODe1jCvOaMcW0hCfyVyv28aJl2PyxxtSF4l_4zs_vnn_XFoTUnpseZDNPmFnb1roT24A_Nav5TcE-9Aj0q0527W1c-UVMa9uhlJOmv8oQ4heaEgSIYr4g7UPY05aYZ59s4y5jjMVrbJ3-8RoGXngxYNx5vDkB2W1y48ShByqVNrbzWKzkM9QkxTTH6QTP4FtvV4Ltssq4DHvfRdqqYiDa0wlvknXGOUKo6BFj0WQD5-5G909zm2d9h3XLVctSUVY8PGtu1_cmGPm52SG5E9r0Kql00I7KaYUlWZt1NNiWVeAzIbqjAXKUNFVpHwQtKqw"  # noqa: E501, S105

    responses.post(
        url=f"{BASE_URL_SKODA}/api/v1/authentication/exchange-authorization-code?tokenType=CONNECT",
        body=dumps(
            {
                "accessToken": access_token,
                "refreshToken": refresh_token,
                "idToken": id_token,
            }
        ),
    )

    session = aiohttp.ClientSession()
    auth = MySkodaAuthorization(session, generate_nonce)

    await auth.authorize("user@example.com", "example")

    assert auth.idk_session is not None
    assert auth.idk_session.access_token == access_token
    assert auth.idk_session.refresh_token == refresh_token
    assert auth.idk_session.id_token == id_token



"""Models for fixtures."""

from dataclasses import dataclass, field
from datetime import datetime
from enum import StrEnum

from mashumaro.mixins.orjson import DataClassORJSONMixin
from mashumaro.mixins.yaml import DataClassYAMLMixin

from myskoda.models.info import Capability, Info


class FixtureReportType(StrEnum):
    GET = "get"


class Endpoint(StrEnum):
    INFO = "info"
    STATUS = "status"
    AIR_CONDITIONING = "air_conditioning"
    AUXILIARY_HEATING = "auxiliary_heating"
    POSITIONS = "positions"
    HEALTH = "health"
    CHARGING = "charging"
    CHARGING_PROFILES = "charging_profiles"
    MAINTENANCE = "maintenance"
    DRIVING_RANGE = "driving_range"
    TRIP_STATISTICS = "trip_statistics"
    DEPARTURE_INFO = "departure_info"
    VEHICLE_CONNECTION_STATUS = "vehicle_connection_status"
    ALL = "all"


@dataclass
class FixtureReportGet(DataClassYAMLMixin):
    type: FixtureReportType
    vehicle_id: int
    success: bool
    endpoint: Endpoint
    raw: str | None = field(default=None)
    url: str | None = field(default=None)
    result: dict | None = field(default=None)
    error: str | None = field(default=None)


@dataclass
class FixtureVehicle(DataClassYAMLMixin):
    id: int
    device_platform: str
    system_model_id: str
    model: str
    model_year: str
    trim_level: str | None
    software_version: str | None
    capabilities: list[Capability]


def create_fixture_vehicle(id: int, info: Info) -> FixtureVehicle:  # noqa: A002
    """Create a new `FixtureVehicle` from an info."""
    return FixtureVehicle(
        id=id,
        device_platform=info.device_platform,
        system_model_id=info.specification.system_model_id,
        capabilities=info.capabilities.capabilities,
        model=info.specification.model,
        model_year=info.specification.model_year,
        software_version=info.software_version,
        trim_level=info.specification.trim_level,
    )


@dataclass
class Fixture(DataClassORJSONMixin, DataClassYAMLMixin):
    """A fixture for a test generated by the CLI."""

    name: str
    description: str | None
    generation_time: datetime
    library_version: str
    vehicles: list[FixtureVehicle]
    reports: list[FixtureReportGet] | None



"""Common constants."""

# Client id extracted from the MySkoda Android app.
CLIENT_ID = "7f045eee-7003-4379-9968-9355ed2adb06@apps_vw-dilab_com"
REDIRECT_URI = "myskoda://redirect/login/"

BASE_URL_SKODA = "https://mysmob.api.connect.skoda-auto.cz"
BASE_URL_IDENT = "https://identity.vwgroup.io"

MQTT_BROKER_HOST = "mqtt.messagehub.de"
MQTT_BROKER_PORT = 8883


MQTT_OPERATION_TIMEOUT = 10 * 60  #  10 minutes
MQTT_OPERATION_TOPICS = [
    "air-conditioning/set-air-conditioning-at-unlock",
    "air-conditioning/set-air-conditioning-seats-heating",
    "air-conditioning/set-air-conditioning-timers",
    "air-conditioning/set-air-conditioning-without-external-power",
    "air-conditioning/set-target-temperature",
    "air-conditioning/start-stop-air-conditioning",
    "auxiliary-heating/start-stop-auxiliary-heating",
    "air-conditioning/start-stop-window-heating",
    "air-conditioning/windows-heating",
    "charging/start-stop-charging",
    "charging/update-battery-support",
    "charging/update-auto-unlock-plug",
    "charging/update-care-mode",
    "charging/update-charge-limit",
    "charging/update-charge-mode",
    "charging/update-charging-profiles",
    "charging/update-charging-current",
    "departure/update-departure-timers",
    "departure/update-minimal-soc",
    "vehicle-access/honk-and-flash",
    "vehicle-access/lock-vehicle",
    "vehicle-services-backup/apply-backup",
    "vehicle-wakeup/wakeup",
]

MQTT_SERVICE_EVENT_TOPICS = [
    "air-conditioning",
    "charging",
    "departure",
    "vehicle-status/access",
    "vehicle-status/lights",
    "vehicle-status/odometer",
]

MQTT_VEHICLE_EVENT_TOPICS = [
    "vehicle-connection-status-update",
    "vehicle-ignition-status",
]

MQTT_ACCOUNT_EVENT_TOPICS = [
    "account-event/privacy",
]
MQTT_KEEPALIVE = 60
MQTT_RECONNECT_DELAY = 5
MQTT_MAX_RECONNECT_DELAY = 600
MQTT_FAST_RETRY = 10
MAX_RETRIES = 5


CACHE_USER_ENDPOINT_IN_HOURS = 6
CACHE_VEHICLE_HEALTH_IN_HOURS = 6

REQUEST_TIMEOUT_IN_SECONDS = 300
DEFAULT_DEBOUNCE_WAIT_SECONDS = 10.0
OPERATION_REFRESH_DELAY_SECONDS = 5.0



"""Models for responses of api/v1/charging/{vin}/history."""

from dataclasses import dataclass, field
from datetime import datetime
from enum import StrEnum

from mashumaro import field_options
from mashumaro.mixins.orjson import DataClassORJSONMixin

from .common import BaseResponse


class ChargingCurrentType(StrEnum):
    AC = "AC"
    DC = "DC"


@dataclass
class ChargingSession(DataClassORJSONMixin):
    start_at: datetime = field(metadata=field_options(alias="startAt"))
    charged_in_kwh: float = field(metadata=field_options(alias="chargedInKWh"))
    duration_in_minutes: int = field(metadata=field_options(alias="durationInMinutes"))
    current_type: ChargingCurrentType = field(metadata=field_options(alias="currentType"))


@dataclass
class ChargingPeriod(DataClassORJSONMixin):
    total_charged_in_kwh: float = field(
        default=0.0, metadata=field_options(alias="totalChargedInKWh")
    )
    sessions: list[ChargingSession] = field(default_factory=list)


@dataclass
class ChargingHistory(BaseResponse):
    next_cursor: datetime | None = field(default=None, metadata=field_options(alias="nextCursor"))
    periods: list[ChargingPeriod] = field(default_factory=list)



"""Models for responses of api/v2/vehicle-status/{vin}."""

from dataclasses import dataclass, field
from datetime import date
from enum import StrEnum

from mashumaro import field_options
from mashumaro.mixins.orjson import DataClassORJSONMixin

from .common import BaseResponse


class VehicleType(StrEnum):
    FUEL = "FUEL"
    HYBRID = "HYBRID"
    ELECTRIC = "ELECTRIC"
    GAS = "GAS"


@dataclass
class StatisticsEntry(DataClassORJSONMixin):
    date: date
    average_fuel_consumption: float | None = field(
        default=None, metadata=field_options(alias="averageFuelConsumption")
    )
    average_gas_consumption: float | None = field(
        default=None, metadata=field_options(alias="averageGasConsumption")
    )
    average_speed_in_kmph: int | None = field(
        default=None, metadata=field_options(alias="averageSpeedInKmph")
    )
    average_electric_consumption: float | None = field(
        default=None, metadata=field_options(alias="averageElectricConsumption")
    )
    average_recuperation: float | None = field(
        default=None, metadata=field_options(alias="averageRecuperation")
    )
    average_aux_consumption: float | None = field(
        default=None, metadata=field_options(alias="averageAuxConsumption")
    )
    mileage_in_km: int | None = field(default=None, metadata=field_options(alias="mileageInKm"))
    travel_time_in_min: int | None = field(
        default=None, metadata=field_options(alias="travelTimeInMin")
    )
    trip_ids: list[int] | None = field(default=None, metadata=field_options(alias="tripIds"))


@dataclass
class TripStatistics(BaseResponse):
    vehicle_type: VehicleType = field(metadata=field_options(alias="vehicleType"))
    detailed_statistics: list[StatisticsEntry] = field(
        metadata=field_options(alias="detailedStatistics")
    )
    overall_average_electric_consumption: float | None = field(
        default=None, metadata=field_options(alias="overallAverageElectricConsumption")
    )
    overall_average_fuel_consumption: float | None = field(
        default=None, metadata=field_options(alias="overallAverageFuelConsumption")
    )
    overall_average_gas_consumption: float | None = field(
        default=None, metadata=field_options(alias="overallAverageGasConsumption")
    )
    overall_average_mileage_in_km: int | None = field(
        default=None, metadata=field_options(alias="overallAverageMileageInKm")
    )
    overall_average_speed_in_kmph: int | None = field(
        default=None, metadata=field_options(alias="overallAverageSpeedInKmph")
    )
    overall_average_travel_time_in_min: int | None = field(
        default=None, metadata=field_options(alias="overallAverageTravelTimeInMin")
    )
    overall_mileage_in_km: int | None = field(
        default=None, metadata=field_options(alias="overallMileageInKm")
    )
    overall_travel_time_in_min: int | None = field(
        default=None, metadata=field_options(alias="overallTravelTimeInMin")
    )


@dataclass
class FuelCost(DataClassORJSONMixin):
    cost: float | None = field(default=None, metadata=field_options(alias="cost"))
    cost_currency: str | None = field(default=None, metadata=field_options(alias="costCurrency"))
    price_per_unit: float | None = field(default=None, metadata=field_options(alias="pricePerUnit"))


@dataclass
class OverallCost(DataClassORJSONMixin):
    total_cost: float | None = field(default=None, metadata=field_options(alias="totalCost"))
    total_cost_currency: str | None = field(
        default=None, metadata=field_options(alias="totalCostCurrency")
    )
    fuel_cost: FuelCost | None = field(default=None, metadata=field_options(alias="fuelCost"))


@dataclass
class Trip(DataClassORJSONMixin):
    id: str | None = field(default=None, metadata=field_options(alias="id"))
    end_time: str | None = field(default=None, metadata=field_options(alias="endTime"))
    start_mileage_in_km: int | None = field(
        default=None, metadata=field_options(alias="startMileageInKm")
    )
    end_mileage_in_km: int | None = field(
        default=None, metadata=field_options(alias="endMileageInKm")
    )
    mileage_in_km: int | None = field(default=None, metadata=field_options(alias="mileageInKm"))
    travel_time_in_min: int | None = field(
        default=None, metadata=field_options(alias="travelTimeInMin")
    )
    average_speed_in_kmph: int | None = field(
        default=None, metadata=field_options(alias="averageSpeedInKmph")
    )
    average_fuel_consumption: float | None = field(
        default=None, metadata=field_options(alias="averageFuelConsumption")
    )
    cost: OverallCost | None = field(default=None, metadata=field_options(alias="cost"))


@dataclass
class DailyTrip(DataClassORJSONMixin):
    date: str
    overall_mileage: int | None = field(
        default=None, metadata=field_options(alias="overallMileage")
    )
    overall_cost: OverallCost | None = field(
        default=None, metadata=field_options(alias="overallCost")
    )
    trips: list[Trip] | None = field(default=None, metadata=field_options(alias="trips"))


@dataclass
class SingleTrips(BaseResponse):
    daily_trips: list[DailyTrip] = field(metadata=field_options(alias="dailyTrips"))
    vehicle_type: VehicleType | None = field(
        default=None, metadata=field_options(alias="vehicleType")
    )
